package com.bursie.harvester.sources;

import rita.wordnet.RiWordnet;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * A WordSource that uses the WordNet database for finding related keywords.
 */
public class WordnetSource implements WordSource {

    private RiWordnet wordnet;

    public WordnetSource() {
        this(new RiWordnet(null));  // Would pass in a PApplet normally, but we here we don't need one
    }

    public WordnetSource(RiWordnet riWordnet) {
        this.wordnet = riWordnet;
        wordnet.ignoreCompoundWords(false);
    }

    /**
     * Finds related keywords given a seed keyword
     *
     * @param seedKeyword A seed keyword, doesn't need to be URL escaped.
     * @return A List of related keywords. Might contain duplicates!
     */
    public List<String> getSuggestedKeywords(String seedKeyword) {
        String pos = wordnet.getBestPos(seedKeyword);
        if (pos == null) {
            return Collections.emptyList();
        }

        List<String> words = new ArrayList<String>();

        // Get max 15 synonyms
        String[] synonyms = wordnet.getAllSynonyms(seedKeyword, pos, 15);
        addWords(words, synonyms);
        String[] alsoSee = wordnet.getAllAlsoSees(seedKeyword, pos);
        addWords(words, alsoSee);
        String[] similar = wordnet.getAllSimilar(seedKeyword, pos);
        addWords(words, similar);
        String[] hyponym = wordnet.getAllHyponyms(seedKeyword, pos);
        addWords(words, hyponym);
        String[] holonym = wordnet.getAllHolonyms(seedKeyword, pos);
        addWords(words, holonym);
        String[] derived = wordnet.getAllDerivedTerms(seedKeyword, pos);
        addWords(words, derived);

        if (words.size() != 0) {
            return words;
        }
        return Collections.emptyList();
    }

    private void addWords(List<String> words, String[] newWords) {
        //Since WordNet methods returns null instead of an empty array, we must check before adding
        if (newWords != null) {
            words.addAll(Arrays.asList(newWords));
        }
    }
}

